@@ -97,7 +97,7 @@ public:
 	void SetConditionData(Macro &m);
 	void SwapActions(Macro *m, int pos1, int pos2);
 	void SwapConditions(Macro *m, int pos1, int pos2);
-	void HighligthMacroSettingsButton(bool enable = true);
+	void HighlightMacroSettingsButton(bool enable = true);
 
 public slots:
 	void on_macroAdd_clicked();
@@ -386,6 +386,6 @@ private:
 };
 
 void OpenSettingsWindow();
-void HighligthMacroSettingsButton(bool enable);
+void HighlightMacroSettingsButton(bool enable);
 
 } // namespace advss
