@@ -181,15 +181,7 @@ void MacroInputEdit::SetInputVariablesAndValues(
 	_variables = inputs;
 	_values = values;
 	if ((size_t)_values.size() < _variables._inputVariables.size()) {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-		for (int i = 0;
-		     _variables._inputVariables.size() - (size_t)_values.size();
-		     i++) {
-			_values.push_back({});
-		}
-#else
 		_values.resize(_variables._inputVariables.size());
-#endif
 	}
 	SetupWidgets();
 }
@@ -199,7 +191,7 @@ bool MacroInputEdit::HasInputsToSet() const
 	return !_variables._inputVariables.empty();
 }
 
-void HighligthMacroSettingsButton(bool enable);
+void HighlightMacroSettingsButton(bool enable);
 
 void MacroInputEdit::SetupWidgets()
 {
