@@ -1,7 +1,8 @@
 #include "macro-settings.hpp"
 #include "layout-helpers.hpp"
-#include "obs-module-helper.hpp"
 #include "macro.hpp"
+#include "obs-module-helper.hpp"
+#include "plugin-state-helpers.hpp"
 
 #include <QDialogButtonBox>
 #include <QScrollArea>
@@ -20,6 +21,10 @@ void GlobalMacroSettings::Save(obs_data_t *obj) const
 	obs_data_set_bool(data, "highlightActions", _highlightActions);
 	obs_data_set_bool(data, "newMacroRegisterHotkey",
 			  _newMacroRegisterHotkeys);
+	obs_data_set_bool(data, "newMacroUseShortCircuitEvaluation",
+			  _newMacroUseShortCircuitEvaluation);
+	obs_data_set_bool(data, "saveSettingsOnMacroChange",
+			  _saveSettingsOnMacroChange);
 	obs_data_set_obj(obj, "macroSettings", data);
 	obs_data_release(data);
 }
@@ -38,6 +43,12 @@ void GlobalMacroSettings::Load(obs_data_t *obj)
 	_highlightActions = obs_data_get_bool(data, "highlightActions");
 	_newMacroRegisterHotkeys =
 		obs_data_get_bool(data, "newMacroRegisterHotkey");
+	_newMacroUseShortCircuitEvaluation =
+		obs_data_get_bool(data, "newMacroUseShortCircuitEvaluation");
+	_saveSettingsOnMacroChange =
+		obs_data_has_user_value(data, "saveSettingsOnMacroChange")
+			? obs_data_get_bool(data, "saveSettingsOnMacroChange")
+			: true;
 	obs_data_release(data);
 }
 
@@ -45,16 +56,29 @@ MacroSettingsDialog::MacroSettingsDialog(QWidget *parent,
 					 const GlobalMacroSettings &settings,
 					 Macro *macro)
 	: QDialog(parent),
-	  _executed(new QCheckBox(obs_module_text(
+	  _highlightExecutedMacros(new QCheckBox(obs_module_text(
 		  "AdvSceneSwitcher.macroTab.highlightExecutedMacros"))),
-	  _conditions(new QCheckBox(obs_module_text(
+	  _highlightConditions(new QCheckBox(obs_module_text(
 		  "AdvSceneSwitcher.macroTab.highlightTrueConditions"))),
-	  _actions(new QCheckBox(obs_module_text(
+	  _highlightActions(new QCheckBox(obs_module_text(
 		  "AdvSceneSwitcher.macroTab.highlightPerformedActions"))),
 	  _newMacroRegisterHotkeys(new QCheckBox(obs_module_text(
 		  "AdvSceneSwitcher.macroTab.newMacroRegisterHotkey"))),
+	  _newMacroUseShortCircuitEvaluation(new QCheckBox(obs_module_text(
+		  "AdvSceneSwitcher.macroTab.newMacroUseShortCircuitEvaluation"))),
+	  _saveSettingsOnMacroChange(new QCheckBox(obs_module_text(
+		  "AdvSceneSwitcher.macroTab.saveSettingsOnMacroChange"))),
 	  _currentMacroRegisterHotkeys(new QCheckBox(obs_module_text(
-		  "AdvSceneSwitcher.macroTab.currentDisableHotkeys"))),
+		  "AdvSceneSwitcher.macroTab.currentRegisterHotkeys"))),
+	  _currentUseShortCircuitEvaluation(new QCheckBox(obs_module_text(
+		  "AdvSceneSwitcher.macroTab.currentUseShortCircuitEvaluation"))),
+	  _currentUseCustomConditionCheckInterval(new QCheckBox(obs_module_text(
+		  "AdvSceneSwitcher.macroTab.currentUseCustomConditionCheckInterval"))),
+	  _currentCustomConditionCheckInterval(
+		  new DurationSelection(this, true, 0.01)),
+	  _currentCustomConditionCheckIntervalWarning(new QLabel(obs_module_text(
+		  "AdvSceneSwitcher.macroTab.currentUseCustomConditionCheckIntervalWarning"))),
+	  _currentPauseSaveBehavior(new QComboBox(this)),
 	  _currentSkipOnStartup(new QCheckBox(obs_module_text(
 		  "AdvSceneSwitcher.macroTab.currentSkipExecutionOnStartup"))),
 	  _currentStopActionsIfNotDone(new QCheckBox(obs_module_text(
@@ -83,12 +107,32 @@ MacroSettingsDialog::MacroSettingsDialog(QWidget *parent,
 	setWindowModality(Qt::WindowModality::WindowModal);
 	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
+	_newMacroUseShortCircuitEvaluation->setToolTip(obs_module_text(
+		"AdvSceneSwitcher.macroTab.shortCircuit.tooltip"));
+	_saveSettingsOnMacroChange->setToolTip(obs_module_text(
+		"AdvSceneSwitcher.macroTab.saveSettingsOnMacroChange.tooltip"));
+	_currentUseShortCircuitEvaluation->setToolTip(obs_module_text(
+		"AdvSceneSwitcher.macroTab.shortCircuit.tooltip"));
+
+	_currentPauseSaveBehavior->addItem(
+		obs_module_text(
+			"AdvSceneSwitcher.macroTab.pauseStateSaveBehavior.persist"),
+		static_cast<int>(Macro::PauseStateSaveBehavior::PERSIST));
+	_currentPauseSaveBehavior->addItem(
+		obs_module_text(
+			"AdvSceneSwitcher.macroTab.pauseStateSaveBehavior.pause"),
+		static_cast<int>(Macro::PauseStateSaveBehavior::PAUSE));
+	_currentPauseSaveBehavior->addItem(
+		obs_module_text(
+			"AdvSceneSwitcher.macroTab.pauseStateSaveBehavior.unpause"),
+		static_cast<int>(Macro::PauseStateSaveBehavior::UNPAUSE));
+
 	auto highlightOptions = new QGroupBox(
 		obs_module_text("AdvSceneSwitcher.macroTab.highlightSettings"));
 	auto highlightLayout = new QVBoxLayout;
-	highlightLayout->addWidget(_executed);
-	highlightLayout->addWidget(_conditions);
-	highlightLayout->addWidget(_actions);
+	highlightLayout->addWidget(_highlightExecutedMacros);
+	highlightLayout->addWidget(_highlightConditions);
+	highlightLayout->addWidget(_highlightActions);
 	highlightOptions->setLayout(highlightLayout);
 
 	auto hotkeyOptions = new QGroupBox(
@@ -101,8 +145,29 @@ MacroSettingsDialog::MacroSettingsDialog(QWidget *parent,
 	auto generalOptions = new QGroupBox(
 		obs_module_text("AdvSceneSwitcher.macroTab.generalSettings"));
 	auto generalLayout = new QVBoxLayout;
+	generalLayout->addWidget(_saveSettingsOnMacroChange);
 	generalLayout->addWidget(_currentSkipOnStartup);
 	generalLayout->addWidget(_currentStopActionsIfNotDone);
+	generalLayout->addWidget(_currentUseShortCircuitEvaluation);
+	generalLayout->addWidget(_newMacroUseShortCircuitEvaluation);
+
+	auto durationLayout = new QHBoxLayout();
+	durationLayout->addWidget(_currentUseCustomConditionCheckInterval);
+	durationLayout->addWidget(_currentCustomConditionCheckInterval);
+	durationLayout->addStretch();
+	auto customConditionIntervalLayout = new QVBoxLayout();
+	customConditionIntervalLayout->addLayout(durationLayout);
+	customConditionIntervalLayout->addWidget(
+		_currentCustomConditionCheckIntervalWarning);
+	generalLayout->addLayout(customConditionIntervalLayout);
+
+	auto pauseStateSaveBehavorLayout = new QHBoxLayout();
+	pauseStateSaveBehavorLayout->addWidget(new QLabel(obs_module_text(
+		"AdvSceneSwitcher.macroTab.pauseStateSaveBehavior")));
+	pauseStateSaveBehavorLayout->addWidget(_currentPauseSaveBehavior);
+	pauseStateSaveBehavorLayout->addStretch();
+	generalLayout->addLayout(pauseStateSaveBehavorLayout);
+
 	generalOptions->setLayout(generalLayout);
 
 	auto inputOptions = new QGroupBox(
@@ -178,6 +243,13 @@ MacroSettingsDialog::MacroSettingsDialog(QWidget *parent,
 		&MacroSettingsDialog::PauseButtonEnableChanged);
 	connect(_currentMacroDockAddStatusLabel, &QCheckBox::stateChanged, this,
 		&MacroSettingsDialog::StatusLabelEnableChanged);
+	connect(_currentUseCustomConditionCheckInterval,
+		&QCheckBox::stateChanged, this, [this](int state) {
+			_currentCustomConditionCheckInterval->setEnabled(state);
+		});
+	connect(_currentCustomConditionCheckInterval,
+		&DurationSelection::DurationChanged, this,
+		[this]() { SetCustomConditionIntervalWarningVisibility(); });
 
 	auto scrollArea = new QScrollArea(this);
 	scrollArea->setWidgetResizable(true);
@@ -198,20 +270,43 @@ MacroSettingsDialog::MacroSettingsDialog(QWidget *parent,
 	dialogLayout->addWidget(buttonbox);
 	setLayout(dialogLayout);
 
-	_executed->setChecked(settings._highlightExecuted);
-	_conditions->setChecked(settings._highlightConditions);
-	_actions->setChecked(settings._highlightActions);
+	_highlightExecutedMacros->setChecked(settings._highlightExecuted);
+	_highlightConditions->setChecked(settings._highlightConditions);
+	_highlightActions->setChecked(settings._highlightActions);
 	_newMacroRegisterHotkeys->setChecked(settings._newMacroRegisterHotkeys);
+	_newMacroUseShortCircuitEvaluation->setChecked(
+		settings._newMacroUseShortCircuitEvaluation);
+	_saveSettingsOnMacroChange->setChecked(
+		settings._saveSettingsOnMacroChange);
 
 	if (!macro || macro->IsGroup()) {
 		hotkeyOptions->hide();
-		generalOptions->hide();
+
+		// General group
+		_currentSkipOnStartup->hide();
+		_currentStopActionsIfNotDone->hide();
+		_currentUseShortCircuitEvaluation->hide();
+		SetLayoutVisible(customConditionIntervalLayout, false);
+		SetLayoutVisible(pauseStateSaveBehavorLayout, false);
+
 		inputOptions->hide();
 		_dockOptions->hide();
 		return;
 	}
 
 	_currentMacroRegisterHotkeys->setChecked(macro->PauseHotkeysEnabled());
+	_currentUseShortCircuitEvaluation->setChecked(
+		macro->ShortCircuitEvaluationEnabled());
+	_currentUseCustomConditionCheckInterval->setChecked(
+		macro->CustomConditionCheckIntervalEnabled());
+	_currentCustomConditionCheckInterval->SetDuration(
+		macro->GetCustomConditionCheckInterval());
+	_currentCustomConditionCheckInterval->setEnabled(
+		macro->CustomConditionCheckIntervalEnabled());
+	SetCustomConditionIntervalWarningVisibility();
+	_currentPauseSaveBehavior->setCurrentIndex(
+		_currentPauseSaveBehavior->findData(
+			static_cast<int>(macro->GetPauseStateSaveBehavior())));
 	_currentSkipOnStartup->setChecked(macro->SkipExecOnStart());
 	_currentStopActionsIfNotDone->setChecked(macro->StopActionsIfNotDone());
 	_currentInputs->SetInputs(macro->GetInputVariables());
@@ -313,6 +408,15 @@ void MacroSettingsDialog::Resize()
 	_dockOptions->updateGeometry();
 }
 
+void MacroSettingsDialog::SetCustomConditionIntervalWarningVisibility() const
+{
+	_currentCustomConditionCheckIntervalWarning->setVisible(
+		_currentUseCustomConditionCheckInterval->isChecked() &&
+		GetIntervalValue() >
+			_currentCustomConditionCheckInterval->GetDuration()
+				.Milliseconds());
+}
+
 bool MacroSettingsDialog::AskForSettings(QWidget *parent,
 					 GlobalMacroSettings &userInput,
 					 Macro *macro)
@@ -322,17 +426,33 @@ bool MacroSettingsDialog::AskForSettings(QWidget *parent,
 	if (dialog.exec() != DialogCode::Accepted) {
 		return false;
 	}
-	userInput._highlightExecuted = dialog._executed->isChecked();
-	userInput._highlightConditions = dialog._conditions->isChecked();
-	userInput._highlightActions = dialog._actions->isChecked();
+	userInput._highlightExecuted =
+		dialog._highlightExecutedMacros->isChecked();
+	userInput._highlightConditions =
+		dialog._highlightConditions->isChecked();
+	userInput._highlightActions = dialog._highlightActions->isChecked();
 	userInput._newMacroRegisterHotkeys =
 		dialog._newMacroRegisterHotkeys->isChecked();
+	userInput._newMacroUseShortCircuitEvaluation =
+		dialog._newMacroUseShortCircuitEvaluation->isChecked();
+	userInput._saveSettingsOnMacroChange =
+		dialog._saveSettingsOnMacroChange->isChecked();
 	if (!macro) {
 		return true;
 	}
 
 	macro->EnablePauseHotkeys(
 		dialog._currentMacroRegisterHotkeys->isChecked());
+	macro->SetShortCircuitEvaluation(
+		dialog._currentUseShortCircuitEvaluation->isChecked());
+	macro->SetCustomConditionCheckIntervalEnabled(
+		dialog._currentUseCustomConditionCheckInterval->isChecked());
+	macro->SetCustomConditionCheckInterval(
+		dialog._currentCustomConditionCheckInterval->GetDuration());
+	macro->SetPauseStateSaveBehavior(
+		static_cast<Macro::PauseStateSaveBehavior>(
+			dialog._currentPauseSaveBehavior->currentData()
+				.toInt()));
 	macro->SetSkipExecOnStart(dialog._currentSkipOnStartup->isChecked());
 	macro->SetStopActionsIfNotDone(
 		dialog._currentStopActionsIfNotDone->isChecked());
