@@ -1,5 +1,6 @@
 #include "advanced-scene-switcher.hpp"
 #include "action-queue.hpp"
+#include "cursor-shape-changer.hpp"
 #include "macro-action-edit.hpp"
 #include "macro-condition-edit.hpp"
 #include "macro-export-import-dialog.hpp"
@@ -86,7 +87,8 @@ bool AdvSceneSwitcher::AddNewMacro(std::shared_ptr<Macro> &res,
 	}
 
 	res = std::make_shared<Macro>(
-		name, GetGlobalMacroSettings()._newMacroRegisterHotkeys);
+		name, GetGlobalMacroSettings()._newMacroRegisterHotkeys,
+		GetGlobalMacroSettings()._newMacroUseShortCircuitEvaluation);
 	return true;
 }
 
@@ -98,9 +100,33 @@ void AdvSceneSwitcher::on_macroAdd_clicked()
 		return;
 	}
 
-	ui->macros->Add(newMacro);
-	addPulse->deleteLater();
-	addPulse = nullptr;
+	if (addPulse) {
+		addPulse->deleteLater();
+		addPulse = nullptr;
+	}
+
+	auto selectedMacro = GetSelectedMacro();
+	if (!selectedMacro) {
+		ui->macros->Add(newMacro);
+		emit MacroAdded(QString::fromStdString(name));
+		return;
+	}
+
+	if (selectedMacro->IsGroup()) {
+		ui->macros->Add(newMacro, selectedMacro);
+		emit MacroAdded(QString::fromStdString(name));
+		return;
+	}
+
+	auto selectedMacroGroup = selectedMacro->Parent();
+	if (!selectedMacroGroup) {
+		ui->macros->Add(newMacro, selectedMacro);
+		emit MacroAdded(QString::fromStdString(name));
+		return;
+	}
+
+	Macro::PrepareMoveToGroup(selectedMacroGroup, newMacro);
+	ui->macros->Add(newMacro, selectedMacro);
 	emit MacroAdded(QString::fromStdString(name));
 }
 
@@ -738,7 +764,10 @@ void AdvSceneSwitcher::MacroSelectionChanged()
 		return;
 	}
 	SetEditMacro(*macro);
-	obs_frontend_save();
+
+	if (GetGlobalMacroSettings()._saveSettingsOnMacroChange) {
+		obs_frontend_save();
+	}
 }
 
 void AdvSceneSwitcher::HighlightOnChange()
@@ -759,6 +788,16 @@ void AdvSceneSwitcher::HighlightOnChange()
 	}
 }
 
+static void resetSegmentHighlights(MacroSegmentList *list)
+{
+	MacroSegmentEdit *widget = nullptr;
+	for (int i = 0; (widget = list->WidgetAt(i)); i++) {
+		if (widget && widget->Data()) {
+			(void)widget->Data()->GetHighlightAndReset();
+		}
+	}
+}
+
 void AdvSceneSwitcher::on_macroSettings_clicked()
 {
 	GlobalMacroSettings prop = GetGlobalMacroSettings();
@@ -767,17 +806,33 @@ void AdvSceneSwitcher::on_macroSettings_clicked()
 	if (!accepted) {
 		return;
 	}
+
 	GetGlobalMacroSettings() = prop;
 	emit HighlightMacrosChanged(prop._highlightExecuted);
+
+	// Reset highlights to avoid all macro segments being highlighted, which
+	// would have been highlighted at least once since the moment the
+	// highlighting was disabled
+	if (prop._highlightConditions) {
+		resetSegmentHighlights(ui->conditionsList);
+	}
+	if (prop._highlightActions) {
+		resetSegmentHighlights(ui->actionsList);
+		resetSegmentHighlights(ui->elseActionsList);
+	}
+
+	SetCheckIntervalTooLowVisibility();
 }
 
 static void moveControlsToSplitter(QSplitter *splitter, int idx,
 				   QLayoutItem *item)
 {
-	static int splitterHandleWidth = 38;
+	static int splitterHandleWidth = 32;
 	auto handle = splitter->handle(idx);
 	auto layout = item->layout();
-	layout->setContentsMargins(7, 7, 7, 7);
+	int leftMargin, rightMargin;
+	layout->getContentsMargins(&leftMargin, nullptr, &rightMargin, nullptr);
+	layout->setContentsMargins(leftMargin, 0, rightMargin, 0);
 	handle->setLayout(layout);
 	splitter->setHandleWidth(splitterHandleWidth);
 	splitter->setStyleSheet("QSplitter::handle {background: transparent;}");
@@ -827,6 +882,32 @@ static void runSegmentHighligtChecks(AdvSceneSwitcher *ss)
 	}
 }
 
+static QToolBar *
+setupToolBar(const std::initializer_list<std::initializer_list<QWidget *>>
+		     &widgetGroups)
+{
+	auto toolbar = new QToolBar();
+	toolbar->setIconSize({16, 16});
+
+	QAction *lastSeperator = nullptr;
+
+	for (const auto &widgetGroup : widgetGroups) {
+		for (const auto &widget : widgetGroup) {
+			toolbar->addWidget(widget);
+		}
+		lastSeperator = toolbar->addSeparator();
+	}
+
+	if (lastSeperator) {
+		toolbar->removeAction(lastSeperator);
+	}
+
+	// Prevent "extension" button from showing up
+	toolbar->setSizePolicy(QSizePolicy::MinimumExpanding,
+			       QSizePolicy::Preferred);
+	return toolbar;
+}
+
 void AdvSceneSwitcher::SetupMacroTab()
 {
 	ui->macroElseActions->installEventFilter(this);
@@ -835,6 +916,11 @@ void AdvSceneSwitcher::SetupMacroTab()
 	if (GetMacros().size() == 0 && !switcher->disableHints) {
 		addPulse = HighlightWidget(ui->macroAdd, QColor(Qt::green));
 	}
+
+	auto macroControls = setupToolBar({{ui->macroAdd, ui->macroRemove},
+					   {ui->macroUp, ui->macroDown}});
+	ui->macroControlLayout->addWidget(macroControls);
+
 	ui->macros->Reset(GetMacros(),
 			  GetGlobalMacroSettings()._highlightExecuted);
 	connect(ui->macros, SIGNAL(MacroSelectionAboutToChange()), this,
@@ -886,13 +972,7 @@ void AdvSceneSwitcher::SetupMacroTab()
 		SLOT(HighlightOnChange()));
 	onChangeHighlightTimer.start();
 
-	// Move condition controls into splitter handle layout
-	moveControlsToSplitter(ui->macroActionConditionSplitter, 1,
-			       ui->macroConditionsLayout->takeAt(1));
-	moveControlsToSplitter(ui->macroElseActionSplitter, 1,
-			       ui->macroActionsLayout->takeAt(1));
-
-	// Set action and condition control icons
+	// Set action and condition toolbars
 	const std::string pathPrefix =
 		GetDataFilePath("res/images/" + GetThemeTypeName());
 	SetButtonIcon(ui->actionTop, (pathPrefix + "DoubleUp.svg").c_str());
@@ -907,6 +987,44 @@ void AdvSceneSwitcher::SetupMacroTab()
 	SetButtonIcon(ui->toggleElseActions,
 		      (pathPrefix + "NotEqual.svg").c_str());
 
+	auto conditionToolbar =
+		setupToolBar({{ui->conditionAdd, ui->conditionRemove},
+			      {ui->conditionTop, ui->conditionUp,
+			       ui->conditionDown, ui->conditionBottom}});
+	auto actionToolbar = setupToolBar({{ui->actionAdd, ui->actionRemove},
+					   {ui->actionTop, ui->actionUp,
+					    ui->actionDown, ui->actionBottom}});
+	auto elseActionToolbar =
+		setupToolBar({{ui->elseActionAdd, ui->elseActionRemove},
+			      {ui->elseActionTop, ui->elseActionUp,
+			       ui->elseActionDown, ui->elseActionBottom}});
+
+	ui->conditionControlsLayout->addWidget(conditionToolbar);
+	ui->actionControlsLayout->insertWidget(0, actionToolbar);
+	ui->elseActionControlsLayout->addWidget(elseActionToolbar);
+
+	// Move condition controls into splitter handle layout
+	moveControlsToSplitter(ui->macroActionConditionSplitter, 1,
+			       ui->macroConditionsLayout->takeAt(1));
+	moveControlsToSplitter(ui->macroElseActionSplitter, 1,
+			       ui->macroActionsLayout->takeAt(1));
+
+	// Override splitter cursor icon when hovering over controls in splitter
+	SetCursorOnWidgetHover(ui->conditionAdd, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->conditionRemove,
+			       Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->conditionTop, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->conditionUp, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->conditionDown, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->conditionBottom,
+			       Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->actionAdd, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->actionRemove, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->actionTop, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->actionUp, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->actionDown, Qt::CursorShape::ArrowCursor);
+	SetCursorOnWidgetHover(ui->actionBottom, Qt::CursorShape::ArrowCursor);
+
 	// Reserve more space for macro edit area than for the macro list
 	ui->macroListMacroEditSplitter->setStretchFactor(0, 1);
 	ui->macroListMacroEditSplitter->setStretchFactor(1, 4);
@@ -981,8 +1099,30 @@ void AdvSceneSwitcher::ShowMacroContextMenu(const QPoint &pos)
 	menu.exec(globalPos);
 }
 
-static void handleCustomLabelChange(MacroSegmentEdit *segmentEdit,
-				    QAction *contextMenuOption)
+static bool handleCustomLabelRename(MacroSegmentEdit *segmentEdit)
+{
+	std::string label;
+	auto segment = segmentEdit->Data();
+	if (!segment) {
+		return false;
+	}
+
+	bool accepted = NameDialog::AskForName(
+		GetSettingsWindow(),
+		obs_module_text(
+			"AdvSceneSwitcher.macroTab.segment.setCustomLabel"),
+		"", label, QString::fromStdString(segment->GetCustomLabel()));
+	if (!accepted) {
+		return false;
+	}
+
+	segment->SetCustomLabel(label);
+	segmentEdit->HeaderInfoChanged("");
+	return true;
+}
+
+static void handleCustomLabelEnableChange(MacroSegmentEdit *segmentEdit,
+					  QAction *contextMenuOption)
 {
 	bool enable = contextMenuOption->isChecked();
 	auto segment = segmentEdit->Data();
@@ -993,22 +1133,93 @@ static void handleCustomLabelChange(MacroSegmentEdit *segmentEdit,
 		return;
 	}
 
-	std::string label;
-	bool accepted = NameDialog::AskForName(
-		GetSettingsWindow(),
-		obs_module_text(
-			"AdvSceneSwitcher.macroTab.segment.setCustomLabel"),
-		"", label, QString::fromStdString(segment->GetCustomLabel()));
-	if (!accepted) {
+	if (!handleCustomLabelRename(segmentEdit)) {
 		segment->SetUseCustomLabel(false);
+	}
+}
+
+static void setupSegmentLabelContextMenuEntries(MacroSegmentEdit *segmentEdit,
+						QMenu &menu)
+{
+	if (!segmentEdit) {
 		return;
 	}
 
-	segment->SetCustomLabel(label);
-	segmentEdit->HeaderInfoChanged("");
+	auto segment = segmentEdit ? segmentEdit->Data() : nullptr;
+	const bool customLabelIsEnabled = segment &&
+					  segment->GetUseCustomLabel();
+
+	auto enableCustomLabel = menu.addAction(obs_module_text(
+		"AdvSceneSwitcher.macroTab.segment.useCustomLabel"));
+	enableCustomLabel->setCheckable(true);
+	enableCustomLabel->setChecked(customLabelIsEnabled);
+	QWidget::connect(enableCustomLabel, &QAction::triggered,
+			 [segmentEdit, enableCustomLabel]() {
+				 handleCustomLabelEnableChange(
+					 segmentEdit, enableCustomLabel);
+			 });
+
+	if (!customLabelIsEnabled) {
+		return;
+	}
+
+	auto customLabelRename = menu.addAction(obs_module_text(
+		"AdvSceneSwitcher.macroTab.segment.customLabelRename"));
+	QWidget::connect(customLabelRename, &QAction::triggered,
+			 [segmentEdit]() {
+				 handleCustomLabelRename(segmentEdit);
+			 });
+}
+
+static void setupCopyPasteContextMenuEnry(AdvSceneSwitcher *ss,
+					  MacroSegmentEdit *segmentEdit,
+					  QMenu &menu)
+{
+	auto copy = menu.addAction(
+		obs_module_text("AdvSceneSwitcher.macroTab.segment.copy"), ss,
+		[ss]() { ss->CopyMacroSegment(); });
+	copy->setEnabled(!!segmentEdit);
+
+	bool pasteAsElseAction = true;
+	const char *pasteText = "AdvSceneSwitcher.macroTab.segment.paste";
+	if (MacroActionIsInClipboard()) {
+		if (IsCursorInWidgetArea(ss->ui->macroActions)) {
+			pasteAsElseAction = false;
+			pasteText =
+				"AdvSceneSwitcher.macroTab.segment.pasteAction";
+		} else if (IsCursorInWidgetArea(ss->ui->macroElseActions)) {
+			pasteAsElseAction = true;
+			pasteText =
+				"AdvSceneSwitcher.macroTab.segment.pasteElseAction";
+		}
+	}
+	auto paste = menu.addAction(
+		obs_module_text(pasteText), ss, [ss, pasteAsElseAction]() {
+			SetCopySegmentTargetActionType(pasteAsElseAction);
+			ss->PasteMacroSegment();
+		});
+	paste->setEnabled(MacroSegmentIsInClipboard());
+}
+
+static void setupRemoveContextMenuEnry(
+	AdvSceneSwitcher *ss,
+	const std::function<void(AdvSceneSwitcher *, int)> &remove,
+	const QPoint &pos, MacroSegmentList *list, QMenu &menu)
+{
+	const auto segmentEditIndex = list->IndexAt(pos);
+	if (segmentEditIndex == -1) {
+		return;
+	}
+
+	menu.addAction(
+		obs_module_text("AdvSceneSwitcher.macroTab.segment.remove"), ss,
+		[ss, remove, segmentEditIndex]() {
+			remove(ss, segmentEditIndex);
+		});
 }
 
 static void setupConextMenu(AdvSceneSwitcher *ss, const QPoint &pos,
+			    std::function<void(AdvSceneSwitcher *, int)> remove,
 			    std::function<void(AdvSceneSwitcher *)> expand,
 			    std::function<void(AdvSceneSwitcher *)> collapse,
 			    std::function<void(AdvSceneSwitcher *)> maximize,
@@ -1018,49 +1229,28 @@ static void setupConextMenu(AdvSceneSwitcher *ss, const QPoint &pos,
 	QMenu menu;
 	auto segmentEdit = list->WidgetAt(pos);
 
-	auto copy = menu.addAction(
-		obs_module_text("AdvSceneSwitcher.macroTab.segment.copy"), ss,
-		[ss]() { ss->CopyMacroSegment(); });
-	copy->setEnabled(!!segmentEdit);
-	auto paste = menu.addAction(
-		obs_module_text("AdvSceneSwitcher.macroTab.segment.paste"), ss,
-		[ss]() { ss->PasteMacroSegment(); });
-	paste->setEnabled(MacroSegmentIsInClipboard());
-
+	setupCopyPasteContextMenuEnry(ss, segmentEdit, menu);
 	menu.addSeparator();
-
-	if (segmentEdit) {
-		auto customLabel = menu.addAction(obs_module_text(
-			"AdvSceneSwitcher.macroTab.segment.useCustomLabel"));
-		customLabel->setCheckable(true);
-		auto segment = segmentEdit ? segmentEdit->Data() : nullptr;
-		customLabel->setChecked(segment &&
-					segment->GetUseCustomLabel());
-		QWidget::connect(customLabel, &QAction::triggered,
-				 std::bind(handleCustomLabelChange, segmentEdit,
-					   customLabel));
-	}
-
+	setupRemoveContextMenuEnry(ss, remove, pos, list, menu);
+	menu.addSeparator();
+	setupSegmentLabelContextMenuEntries(segmentEdit, menu);
 	menu.addSeparator();
-
 	menu.addAction(obs_module_text("AdvSceneSwitcher.macroTab.expandAll"),
 		       ss, [ss, expand]() { expand(ss); });
 	menu.addAction(obs_module_text("AdvSceneSwitcher.macroTab.collapseAll"),
 		       ss, [ss, collapse]() { collapse(ss); });
-
 	menu.addSeparator();
-
 	menu.addAction(obs_module_text("AdvSceneSwitcher.macroTab.maximize"),
 		       ss, [ss, maximize]() { maximize(ss); });
 	menu.addAction(obs_module_text("AdvSceneSwitcher.macroTab.minimize"),
 		       ss, [ss, minimize]() { minimize(ss); });
-
 	menu.exec(list->mapToGlobal(pos));
 }
 
 void AdvSceneSwitcher::ShowMacroActionsContextMenu(const QPoint &pos)
 {
-	setupConextMenu(this, pos, &AdvSceneSwitcher::ExpandAllActions,
+	setupConextMenu(this, pos, &AdvSceneSwitcher::RemoveMacroAction,
+			&AdvSceneSwitcher::ExpandAllActions,
 			&AdvSceneSwitcher::CollapseAllActions,
 			&AdvSceneSwitcher::MaximizeActions,
 			&AdvSceneSwitcher::MinimizeActions, ui->actionsList);
@@ -1068,7 +1258,8 @@ void AdvSceneSwitcher::ShowMacroActionsContextMenu(const QPoint &pos)
 
 void AdvSceneSwitcher::ShowMacroElseActionsContextMenu(const QPoint &pos)
 {
-	setupConextMenu(this, pos, &AdvSceneSwitcher::ExpandAllElseActions,
+	setupConextMenu(this, pos, &AdvSceneSwitcher::RemoveMacroElseAction,
+			&AdvSceneSwitcher::ExpandAllElseActions,
 			&AdvSceneSwitcher::CollapseAllElseActions,
 			&AdvSceneSwitcher::MaximizeElseActions,
 			&AdvSceneSwitcher::MinimizeElseActions,
@@ -1077,7 +1268,8 @@ void AdvSceneSwitcher::ShowMacroElseActionsContextMenu(const QPoint &pos)
 
 void AdvSceneSwitcher::ShowMacroConditionsContextMenu(const QPoint &pos)
 {
-	setupConextMenu(this, pos, &AdvSceneSwitcher::ExpandAllConditions,
+	setupConextMenu(this, pos, &AdvSceneSwitcher::RemoveMacroCondition,
+			&AdvSceneSwitcher::ExpandAllConditions,
 			&AdvSceneSwitcher::CollapseAllConditions,
 			&AdvSceneSwitcher::MaximizeConditions,
 			&AdvSceneSwitcher::MinimizeConditions,
@@ -1099,7 +1291,7 @@ void AdvSceneSwitcher::CopyMacro()
 	}
 
 	OBSDataAutoRelease data = obs_data_create();
-	macro->Save(data);
+	macro->Save(data, true);
 	newMacro->Load(data);
 	newMacro->PostLoad();
 	newMacro->SetName(name);
@@ -1107,8 +1299,10 @@ void AdvSceneSwitcher::CopyMacro()
 	Macro::PrepareMoveToGroup(macro->Parent(), newMacro);
 
 	ui->macros->Add(newMacro, macro);
-	addPulse->deleteLater();
-	addPulse = nullptr;
+	if (addPulse) {
+		addPulse->deleteLater();
+		addPulse = nullptr;
+	}
 	emit MacroAdded(QString::fromStdString(name));
 }
 
@@ -1400,7 +1594,7 @@ static void fade(QWidget *widget, bool fadeOut)
 	animation->start(QPropertyAnimation::DeleteWhenStopped);
 }
 
-void fadeWidgets(const std::vector<QWidget *> &widgets, bool fadeOut)
+static void fadeWidgets(const std::vector<QWidget *> &widgets, bool fadeOut)
 {
 	for (const auto &widget : widgets) {
 		fade(widget, fadeOut);
