@@ -160,7 +160,7 @@ void MacroTreeItem::Update(bool force)
 		_spacer = nullptr;
 	}
 
-	if (_type == Type::Group) {
+	if (_type == Type::Group && _expand) {
 		_boxLayout->removeWidget(_expand);
 		_expand->deleteLater();
 		_expand = nullptr;
@@ -178,6 +178,7 @@ void MacroTreeItem::Update(bool force)
 				       QSizePolicy::Maximum);
 		_expand->setMaximumSize(10, 16);
 		_expand->setMinimumSize(10, 0);
+		_expand->setProperty("class", "checkbox-icon indicator-expand");
 #ifdef __APPLE__
 		_expand->setAttribute(Qt::WA_LayoutUsesWidgetRect);
 #endif
@@ -888,14 +889,7 @@ void MacroTree::dropEvent(QDropEvent *event)
 	QModelIndexList indices = selectedIndexes();
 
 	DropIndicatorPosition indicator = dropIndicatorPosition();
-	int row = indexAt(
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-			  event->position().toPoint()
-#else
-			  event->pos()
-#endif
-				  )
-			  .row();
+	int row = indexAt(event->position().toPoint()).row();
 	bool emptyDrop = row == -1;
 
 	if (emptyDrop) {
