@@ -1,11 +1,11 @@
 #pragma once
 #include "export-symbol-helper.hpp"
 
+#include <QAbstractButton>
 #include <QColor>
 #include <QComboBox>
 #include <QIcon>
 #include <QListWidget>
-#include <QPushButton>
 #include <QString>
 #include <QWidget>
 #include <string>
@@ -19,7 +19,7 @@ EXPORT QObject *HighlightWidget(QWidget *widget, QColor startColor,
 				bool once = false);
 
 EXPORT void SetHeightToContentHeight(QListWidget *list);
-EXPORT void SetButtonIcon(QPushButton *button, const char *path);
+EXPORT void SetButtonIcon(QAbstractButton *button, const char *path);
 EXPORT int
 FindIdxInRagne(QComboBox *list, int start, int stop, const std::string &value,
 	       Qt::MatchFlags = Qt::MatchExactly | Qt::MatchCaseSensitive);
@@ -30,7 +30,10 @@ EXPORT void DisplayTrayMessage(const QString &title, const QString &msg,
 			       const QIcon &icon = QIcon());
 
 EXPORT std::string GetThemeTypeName();
+EXPORT QWidget *GetSettingsWindow();
 
 void QeueUITask(void (*task)(void *param), void *param);
 
+bool IsCursorInWidgetArea(QWidget *widget);
+
 } // namespace advss
