@@ -176,15 +176,26 @@ bool MacroConditionDate::CheckRegularDate(int64_t msSinceLastCheck)
 
 bool MacroConditionDate::CheckCondition()
 {
-	auto m = GetMacro();
-	if (!m) {
+	auto macro = GetMacro();
+	if (!macro) {
 		return false;
 	}
-	auto msSinceLastCheck = MillisecondsSinceMacroConditionCheck(m);
+
+	const auto now = std::chrono::high_resolution_clock::now();
+	const auto lastCheck = LastMacroConditionCheckTime(macro);
+
+	const auto timePassed = now - lastCheck;
+
+	const auto msSinceLastCheck =
+		MacroWasCheckedSinceLastStart(macro)
+			? std::chrono::duration_cast<std::chrono::milliseconds>(
+				  timePassed)
+			: std::chrono::milliseconds(0);
+
 	if (_dayOfWeekCheck) {
-		return CheckDayOfWeek(msSinceLastCheck);
+		return CheckDayOfWeek(msSinceLastCheck.count());
 	}
-	return CheckRegularDate(msSinceLastCheck);
+	return CheckRegularDate(msSinceLastCheck.count());
 }
 
 bool MacroConditionDate::Save(obs_data_t *obj) const
