@@ -139,7 +139,7 @@ void TwitchCategorySelection::SelectionChanged(int index)
 {
 	TwitchCategory category{itemData(index).toInt(),
 				currentText().toStdString()};
-	emit CategoreyChanged(category);
+	emit CategoryChanged(category);
 }
 
 std::map<QString, int> CategoryGrabber::_categoryMap = {};
@@ -193,7 +193,7 @@ void CategoryGrabber::Search(const std::string &)
 	httplib::Params params = {{"first", "100"},
 				  {"after", cursor},
 				  {"query", _searchString}};
-	auto response = SendGetRequest(*_token, uri, path, params);
+	auto response = SendGetRequest(*_token, uri, path, params, true);
 
 	while (response.status == 200 && !_stop) {
 		cursor = ParseReply(response.data);
@@ -203,7 +203,7 @@ void CategoryGrabber::Search(const std::string &)
 		params = {{"first", "100"},
 			  {"after", cursor},
 			  {"query", _searchString}};
-		response = SendGetRequest(*_token, uri, path, params);
+		response = SendGetRequest(*_token, uri, path, params, true);
 		emit CategoryCountUpdated(_categoryMap.size() - startCount);
 	}
 }
@@ -217,7 +217,7 @@ void CategoryGrabber::GetAll()
 	static std::string cursor;
 
 	httplib::Params params = {{"first", "100"}, {"after", cursor}};
-	auto response = SendGetRequest(*_token, uri, path, params);
+	auto response = SendGetRequest(*_token, uri, path, params, true);
 
 	while (response.status == 200 && !_stop) {
 		cursor = ParseReply(response.data);
@@ -225,7 +225,7 @@ void CategoryGrabber::GetAll()
 			break; // End of category list
 		}
 		params = {{"first", "100"}, {"after", cursor}};
-		response = SendGetRequest(*_token, uri, path, params);
+		response = SendGetRequest(*_token, uri, path, params, true);
 		emit CategoryCountUpdated(_categoryMap.size());
 	}
 }
@@ -364,8 +364,8 @@ TwitchCategoryWidget::TwitchCategoryWidget(QWidget *parent)
 	  _manualSearch(new TwitchCategorySearchButton(this))
 {
 	QWidget::connect(_selection,
-			 SIGNAL(CategoreyChanged(const TwitchCategory &)), this,
-			 SIGNAL(CategoreyChanged(const TwitchCategory &)));
+			 SIGNAL(CategoryChanged(const TwitchCategory &)), this,
+			 SIGNAL(CategoryChanged(const TwitchCategory &)));
 
 	auto layout = new QHBoxLayout();
 	layout->setContentsMargins(0, 0, 0, 0);
