@@ -3,38 +3,65 @@
 #include "twitch-helpers.hpp"
 
 #include <log-helper.hpp>
-#include <plugin-state-helpers.hpp>
 
 #undef DispatchMessage
 
 namespace advss {
 
+using namespace std::chrono_literals;
+
 using websocketpp::lib::placeholders::_1;
 using websocketpp::lib::placeholders::_2;
 using websocketpp::lib::bind;
 
-static const int reconnectDelay = 15;
+static const auto reconnectDelay = 15s;
 
 /* ------------------------------------------------------------------------- */
 
-static ParsedTags parseTags(const std::string &tags)
+static std::vector<IRCMessage::Badge> parseBadgeTag(const std::string &tag)
+{
+	std::vector<std::string> badgePairs;
+	size_t badgePos = 0;
+	size_t badgeEndPos;
+
+	while ((badgeEndPos = tag.find(',', badgePos)) != std::string::npos) {
+		badgePairs.push_back(
+			tag.substr(badgePos, badgeEndPos - badgePos));
+		badgePos = badgeEndPos + 1;
+	}
+
+	badgePairs.push_back(tag.substr(badgePos));
+
+	std::vector<IRCMessage::Badge> badges;
+	for (const auto &badgePair : badgePairs) {
+		size_t slashPos = badgePair.find('/');
+		const auto badgeName = badgePair.substr(0, slashPos);
+		const auto badgeValue = (slashPos != std::string::npos)
+						? badgePair.substr(slashPos + 1)
+						: "";
+		badges.push_back({badgeName, badgeValue != "0"});
+	}
+
+	return badges;
+}
+
+static void parseTags(const std::string &tags, IRCMessage &message)
 {
-	ParsedTags parsedTags;
 	static constexpr std::array<std::string_view, 2> tagsToIgnore = {
 		"client-nonce", "flags"};
 
-	std::vector<std::string> parsedTagPairs;
+	std::vector<std::string> properties;
 	size_t pos = 0;
 	size_t endPos;
 
 	while ((endPos = tags.find(';', pos)) != std::string::npos) {
-		parsedTagPairs.push_back(tags.substr(pos, endPos - pos));
+		properties.push_back(tags.substr(pos, endPos - pos));
 		pos = endPos + 1;
 	}
 
-	parsedTagPairs.push_back(tags.substr(pos));
+	properties.push_back(tags.substr(pos));
 
-	for (const auto &tagPair : parsedTagPairs) {
+	for (const auto &tagPair : properties) {
 		size_t equalsPos = tagPair.find('=');
 		std::string tagName = tagPair.substr(0, equalsPos);
 		std::string tagValue = (equalsPos != std::string::npos)
@@ -47,113 +74,58 @@ static ParsedTags parseTags(const std::string &tags)
 		}
 
 		if (tagValue.empty()) {
-			parsedTags.tagMap[tagName] = {};
 			continue;
 		}
 
-		if (tagName == "badges" || tagName == "badge-info") {
-			ParsedTags::BadgeMap badgeMap;
-			std::vector<std::string> badgePairs;
-			size_t badgePos = 0;
-			size_t badgeEndPos;
-
-			while ((badgeEndPos = tagValue.find(',', badgePos)) !=
-			       std::string::npos) {
-				badgePairs.push_back(tagValue.substr(
-					badgePos, badgeEndPos - badgePos));
-				badgePos = badgeEndPos + 1;
-			}
-
-			badgePairs.push_back(tagValue.substr(badgePos));
-
-			for (const auto &badgePair : badgePairs) {
-				size_t slashPos = badgePair.find('/');
-				std::string badgeName =
-					badgePair.substr(0, slashPos);
-				std::string badgeValue =
-					(slashPos != std::string::npos)
-						? badgePair.substr(slashPos + 1)
-						: "";
-
-				badgeMap[badgeName] = badgeValue;
-			}
-
-			parsedTags.tagMap[tagName] = badgeMap;
-
+		if (tagName == "badge-info") {
+			message.properties.badgeInfoString = tagValue;
+		} else if (tagName == "badges") {
+			message.properties.badges = parseBadgeTag(tagValue);
+			message.properties.badgesString = tagValue;
+		} else if (tagName == "bits") {
+			message.properties.bits = std::stoi(tagValue);
+		} else if (tagName == "color") {
+			message.properties.color = tagValue;
+		} else if (tagName == "display-name") {
+			message.properties.displayName = tagValue;
+		} else if (tagName == "emote-only") {
+			message.properties.isUsingOnlyEmotes = tagValue == "1";
 		} else if (tagName == "emotes") {
-			ParsedTags::EmoteMap emoteMap;
-			std::vector<std::string> emotePairs;
-			size_t emotePos = 0;
-			size_t emoteEndPos;
-
-			while ((emoteEndPos = tagValue.find('/', emotePos)) !=
-			       std::string::npos) {
-				emotePairs.push_back(tagValue.substr(
-					emotePos, emoteEndPos - emotePos));
-				emotePos = emoteEndPos + 1;
-			}
-
-			emotePairs.push_back(tagValue.substr(emotePos));
-
-			for (const auto &emotePair : emotePairs) {
-				size_t colonPos = emotePair.find(':');
-				std::string emoteId =
-					emotePair.substr(0, colonPos);
-				std::string positions =
-					(colonPos != std::string::npos)
-						? emotePair.substr(colonPos + 1)
-						: "";
-
-				std::vector<std::pair<int, int>> textPositions;
-				size_t positionPos = 0;
-				size_t positionEndPos;
-
-				while ((positionEndPos = positions.find(
-						',', positionPos)) !=
-				       std::string::npos) {
-					std::string position = positions.substr(
-						positionPos,
-						positionEndPos - positionPos);
-					size_t dashPos = position.find('-');
-					int startPos = std::stoi(
-						position.substr(0, dashPos));
-					int endPos = std::stoi(
-						position.substr(dashPos + 1));
-					textPositions.push_back(
-						{startPos, endPos});
-					positionPos = positionEndPos + 1;
-				}
-
-				textPositions.push_back(
-					{std::stoi(
-						 positions.substr(positionPos)),
-					 std::stoi(positions.substr(
-						 positionPos))});
-				emoteMap[emoteId] = textPositions;
-			}
-
-			parsedTags.tagMap[tagName] = emoteMap;
-		} else if (tagName == "emote-sets") {
-			ParsedTags::EmoteSet emoteSetIds;
-			size_t setIdPos = 0;
-			size_t setIdEndPos;
-
-			while ((setIdEndPos = tagValue.find(',', setIdPos)) !=
-			       std::string::npos) {
-				emoteSetIds.push_back(tagValue.substr(
-					setIdPos, setIdEndPos - setIdPos));
-				setIdPos = setIdEndPos + 1;
-			}
-
-			emoteSetIds.push_back(tagValue.substr(setIdPos));
-
-			parsedTags.tagMap[tagName] = emoteSetIds;
-		} else {
-			parsedTags.tagMap[tagName] = tagValue;
+			message.properties.emotesString = tagValue;
+		} else if (tagName == "first-msg") {
+			message.properties.isFirstMessage = tagValue == "1";
+		} else if (tagName == "id") {
+			message.properties.id = tagValue;
+		} else if (tagName == "mod") {
+			message.properties.isMod = tagValue == "1";
+		} else if (tagName == "reply-parent-msg-body") {
+			message.properties.replyParentBody = tagValue;
+		} else if (tagName == "reply-parent-display-name") {
+			message.properties.replyParentDisplayName = tagValue;
+		} else if (tagName == "reply-parent-msg-id") {
+			message.properties.replyParentId = tagValue;
+		} else if (tagName == "reply-parent-user-id") {
+			message.properties.replyParentUserId = tagValue;
+		} else if (tagName == "reply-parent-user-login") {
+			message.properties.replyParentUserLogin = tagValue;
+		} else if (tagName == "reply-thread-parent-msg-id") {
+			message.properties.rootParentId = tagValue;
+		} else if (tagName == "reply-thread-parent-user-login") {
+			message.properties.rootParentUserLogin = tagValue;
+		} else if (tagName == "subscriber") {
+			message.properties.isSubscriber = tagValue == "1";
+		} else if (tagName == "tmi-sent-ts") {
+			message.properties.timestamp = std::stoull(tagValue);
+		} else if (tagName == "turbo") {
+			message.properties.isTurbo = tagValue == "1";
+		} else if (tagName == "user-id") {
+			message.properties.userId = tagValue;
+		} else if (tagName == "user-type") {
+			message.properties.userType = tagValue;
+		} else if (tagName == "vip") {
+			message.properties.isVIP = tagValue == "1";
 		}
 	}
-
-	return parsedTags;
 }
 
 static void parseSource(const std::string &rawSourceComponent,
@@ -209,8 +181,6 @@ static void parseCommand(const std::string &rawCommandComponent,
 		return;
 	} else if (message.command.command == "PING" ||
 		   message.command.command == "001" ||
-		   message.command.command == "JOIN" ||
-		   message.command.command == "PART" ||
 		   message.command.command == "NOTICE" ||
 		   message.command.command == "CLEARCHAT" ||
 		   message.command.command == "HOSTTARGET" ||
@@ -219,6 +189,18 @@ static void parseCommand(const std::string &rawCommandComponent,
 			return;
 		}
 		message.command.parameters = commandParts[1];
+	} else if (message.command.command == "JOIN") {
+		if (commandParts.size() < 2) {
+			return;
+		}
+		message.properties.joinedChannel = true;
+		message.command.parameters = commandParts[1];
+	} else if (message.command.command == "PART") {
+		if (commandParts.size() < 2) {
+			return;
+		}
+		message.properties.leftChannel = true;
+		message.command.parameters = commandParts[1];
 	} else if (message.command.command == "GLOBALUSERSTATE" ||
 		   message.command.command == "USERSTATE" ||
 		   message.command.command == "ROOMSTATE" ||
@@ -271,7 +253,7 @@ static IRCMessage parseMessage(const std::string &message)
 		rawMessageComponent = message.substr(idx);
 	}
 
-	parsedMessage.tags = parseTags(rawTagsComponent);
+	parseTags(rawTagsComponent, parsedMessage);
 	parseSource(rawSourceComponent, parsedMessage);
 	parseCommand(rawCommandComponent, parsedMessage);
 	parsedMessage.message = rawMessageComponent;
@@ -390,15 +372,14 @@ TwitchChatConnection::GetChatConnection(const TwitchToken &token_,
 
 void TwitchChatConnection::ConnectThread()
 {
-	while (!_disconnect) {
-		std::unique_lock<std::mutex> lock(_waitMtx);
+	while (!_stop) {
+		_state = State::CONNECTING;
 		_client.reset();
-		_connected = true;
 		websocketpp::lib::error_code ec;
 		websocketpp::client<websocketpp::config::asio_tls_client>::
 			connection_ptr con = _client.get_connection(_url, ec);
 		if (ec) {
-			blog(LOG_INFO, "Twitch TwitchChatConnection failed: %s",
+			blog(LOG_INFO, "TwitchChatConnection failed: %s",
 			     ec.message().c_str());
 		} else {
 			_client.connect(con);
@@ -406,52 +387,68 @@ void TwitchChatConnection::ConnectThread()
 			_client.run();
 		}
 
+		if (_stop) {
+			break;
+		}
+
 		blog(LOG_INFO,
-		     "Twitch TwitchChatConnection trying to reconnect to in %d seconds.",
-		     reconnectDelay);
+		     "TwitchChatConnection trying to reconnect to in %ld seconds.",
+		     (long int)reconnectDelay.count());
+		std::unique_lock<std::mutex> lock(_waitMtx);
 		_cv.wait_for(lock, std::chrono::seconds(reconnectDelay));
 	}
-	_connected = false;
+	_state = State::DISCONNECTED;
 }
 
 void TwitchChatConnection::Connect()
 {
 	std::lock_guard<std::mutex> lock(_connectMtx);
-	if (_connected) {
+	if (_state == State::CONNECTING || _state == State::CONNECTED) {
 		vblog(LOG_INFO,
 		      "Twitch TwitchChatConnection connect already in progress");
 		return;
 	}
-	_disconnect = true;
+
 	if (_thread.joinable()) {
 		_thread.join();
 	}
-	_disconnect = false;
+
+	_stop = false;
+	_state = State::CONNECTING;
 	_thread = std::thread(&TwitchChatConnection::ConnectThread, this);
 }
 
 void TwitchChatConnection::Disconnect()
 {
 	std::lock_guard<std::mutex> lock(_connectMtx);
-	_disconnect = true;
+	if (_state == State::DISCONNECTED) {
+		vblog(LOG_INFO, "TwitchChatConnection already disconnected");
+		return;
+	}
+
+	_stop = true;
+
 	websocketpp::lib::error_code ec;
 	_client.close(_connection, websocketpp::close::status::normal,
 		      "Twitch chat connection stopping", ec);
+	if (ec) {
+		blog(LOG_INFO, "TwitchChatConnection close failed: %s",
+		     ec.message().c_str());
+
+		// TODO: avoid using stop()
+		_stop = true;
+		std::this_thread::sleep_for(3s);
+		_client.stop();
+	}
+
 	{
 		std::unique_lock<std::mutex> waitLock(_waitMtx);
 		_cv.notify_all();
 	}
 
-	while (_connected) {
-		std::this_thread::sleep_for(std::chrono::milliseconds(10));
-		_client.close(_connection, websocketpp::close::status::normal,
-			      "Twitch chat connection stopping", ec);
-	}
-
 	if (_thread.joinable()) {
 		_thread.join();
 	}
-	_connected = false;
 }
 
 static std::string toLowerCase(const std::string &str)
@@ -468,7 +465,7 @@ static std::string toLowerCase(const std::string &str)
 
 void TwitchChatConnection::ConnectToChat()
 {
-	if (!_connected) {
+	if (_state == State::DISCONNECTED) {
 		Connect();
 		return;
 	}
@@ -517,12 +514,34 @@ void TwitchChatConnection::JoinChannel(const std::string &channelName)
 	Send("JOIN #" + channelName);
 }
 
+static bool nickMatchesTokenUser(const std::string &nick, TwitchToken &token)
+{
+	auto tokenUserName = token.GetName();
+	std::transform(tokenUserName.begin(), tokenUserName.end(),
+		       tokenUserName.begin(),
+		       [](unsigned char c) { return std::tolower(c); });
+	return nick == tokenUserName;
+}
+
 void TwitchChatConnection::HandleJoin(const IRCMessage &message)
 {
+	if (!nickMatchesTokenUser(message.source.nick, _token)) {
+		_messageDispatcher.DispatchMessage(message);
+		return;
+	}
 	_joinedChannelName = std::get<std::string>(message.command.parameters);
 	vblog(LOG_INFO, "Twitch chat join was successful!");
 }
 
+void TwitchChatConnection::HandlePart(const IRCMessage &message)
+{
+	if (!nickMatchesTokenUser(message.source.nick, _token)) {
+		_messageDispatcher.DispatchMessage(message);
+		return;
+	}
+	vblog(LOG_INFO, "Left Twitch chat!");
+}
+
 void TwitchChatConnection::HandleNewMessage(const IRCMessage &message)
 {
 	_messageDispatcher.DispatchMessage(message);
@@ -556,15 +575,24 @@ void TwitchChatConnection::HandleNotice(const IRCMessage &message) const
 void TwitchChatConnection::HandleReconnect()
 {
 	blog(LOG_INFO,
-	     "Received RECONNECT notice! Twitch chat connection will be terminated!");
-	Disconnect();
+	     "Received RECONNECT notice! Twitch chat connection will be terminated shortly!");
+
+	// We need a separate thread here as we cannot close the current
+	// connection from within the message handler
+	std::thread reconnectThread([token = this->_token,
+				     channel = this->_channel]() {
+		auto chatConnection =
+			TwitchChatConnection::GetChatConnection(token, channel);
+		chatConnection->Disconnect();
+		chatConnection->Connect();
+	});
+	reconnectThread.detach();
 }
 
 void TwitchChatConnection::OnOpen(connection_hdl)
 {
 	vblog(LOG_INFO, "Twitch chat connection opened");
-	_connected = true;
-
+	_state = State::CONNECTED;
 	Authenticate();
 }
 
@@ -573,13 +601,14 @@ void TwitchChatConnection::OnMessage(
 	websocketpp::client<websocketpp::config::asio_tls_client>::message_ptr
 		message)
 {
-	constexpr std::string_view authOKCommand = "001";
-	constexpr std::string_view pingCommand = "PING";
-	constexpr std::string_view joinOKCommand = "JOIN";
-	constexpr std::string_view noticeCommand = "NOTICE";
-	constexpr std::string_view reconnectCommand = "RECONNECT";
-	constexpr std::string_view newMessageCommand = "PRIVMSG";
-	constexpr std::string_view whisperCommand = "WHISPER";
+	static constexpr std::string_view authOKCommand = "001";
+	static constexpr std::string_view pingCommand = "PING";
+	static constexpr std::string_view joinCommand = "JOIN";
+	static constexpr std::string_view partCommand = "PART";
+	static constexpr std::string_view noticeCommand = "NOTICE";
+	static constexpr std::string_view reconnectCommand = "RECONNECT";
+	static constexpr std::string_view newMessageCommand = "PRIVMSG";
+	static constexpr std::string_view whisperCommand = "WHISPER";
 
 	if (!message) {
 		return;
@@ -589,7 +618,7 @@ void TwitchChatConnection::OnMessage(
 	}
 
 	std::string payload = message->get_payload();
-	auto messages = parseMessages(payload);
+	const auto messages = parseMessages(payload);
 
 	for (const auto &message : messages) {
 		if (message.command.command == authOKCommand) {
@@ -600,8 +629,10 @@ void TwitchChatConnection::OnMessage(
 		} else if (message.command.command == pingCommand) {
 			Send("PONG " +
 			     std::get<std::string>(message.command.parameters));
-		} else if (message.command.command == joinOKCommand) {
+		} else if (message.command.command == joinCommand) {
 			HandleJoin(message);
+		} else if (message.command.command == partCommand) {
+			HandlePart(message);
 		} else if (message.command.command == newMessageCommand) {
 			HandleNewMessage(message);
 		} else if (message.command.command == whisperCommand) {
@@ -620,7 +651,6 @@ void TwitchChatConnection::OnClose(connection_hdl hdl)
 		con = _client.get_con_from_hdl(hdl);
 	auto msg = con->get_ec().message();
 	blog(LOG_INFO, "Twitch chat connection closed: %s", msg.c_str());
-	_connected = false;
 }
 
 void TwitchChatConnection::OnFail(connection_hdl hdl)
@@ -629,7 +659,6 @@ void TwitchChatConnection::OnFail(connection_hdl hdl)
 		con = _client.get_con_from_hdl(hdl);
 	auto msg = con->get_ec().message();
 	blog(LOG_INFO, "Twitch chat connection failed: %s", msg.c_str());
-	_connected = false;
 }
 
 void TwitchChatConnection::Send(const std::string &msg)
