@@ -257,65 +257,15 @@ void MacroConditionTwitch::ResetChatConnection()
 	_chatConnection.reset();
 }
 
-static void
-setTempVarsHelper(const std::string &jsonStr,
-		  std::function<void(const char *, const char *)> setVar)
-{
-	try {
-		auto json = nlohmann::json::parse(jsonStr);
-		for (auto it = json.begin(); it != json.end(); ++it) {
-			if (it->is_string()) {
-				setVar(it.key().c_str(),
-				       it->get<std::string>().c_str());
-				continue;
-			}
-			setVar(it.key().c_str(), it->dump().c_str());
-		}
-	} catch (const nlohmann::json::exception &e) {
-		vblog(LOG_INFO, "%s", jsonStr.c_str());
-		vblog(LOG_INFO, "%s", e.what());
-	}
-}
-
-static void
-setTempVarsHelper(obs_data_t *data,
-		  std::function<void(const char *, const char *)> setVar)
-{
-	auto jsonStr = obs_data_get_json(data);
-	if (!jsonStr) {
-		return;
-	}
-
-	setTempVarsHelper(jsonStr, setVar);
-}
-
 bool MacroConditionTwitch::CheckChannelGenericEvents()
 {
-	if (!_eventBuffer) {
-		return false;
-	}
-
-	while (!_eventBuffer->Empty()) {
-		auto event = _eventBuffer->ConsumeMessage();
-		if (!event) {
-			continue;
-		}
-		if (_subscriptionID != event->id) {
-			continue;
-		}
-		SetVariableValue(event->ToString());
-		setTempVarsHelper(event->data,
-				  [this](const char *id, const char *value) {
-					  SetTempVarValue(id, value);
-				  });
-
-		if (_clearBufferOnMatch) {
-			_eventBuffer->Clear();
-		}
-		return true;
-	}
-
-	return false;
+	return HandleMatchingSubscriptionEvents([this](const Event &event) {
+		SetVariableValue(event.ToString());
+		SetJsonTempVars(event.data,
+				[this](const char *id, const char *value) {
+					SetTempVarValue(id, value);
+				});
+	});
 }
 
 bool MacroConditionTwitch::CheckChannelLiveEvents()
@@ -345,10 +295,105 @@ bool MacroConditionTwitch::CheckChannelLiveEvents()
 		}
 
 		SetVariableValue(event->ToString());
-		setTempVarsHelper(event->data,
-				  [this](const char *id, const char *value) {
-					  SetTempVarValue(id, value);
-				  });
+		SetJsonTempVars(event->data,
+				[this](const char *id, const char *value) {
+					SetTempVarValue(id, value);
+				});
+
+		if (_clearBufferOnMatch) {
+			_eventBuffer->Clear();
+		}
+		return true;
+	}
+
+	return false;
+}
+
+bool MacroConditionTwitch::CheckChannelRewardChangeEvents()
+{
+	return HandleMatchingSubscriptionEvents([this](const Event &event) {
+		SetVariableValue(event.ToString());
+		SetJsonTempVars(event.data,
+				[this](const char *id, const char *value) {
+					SetTempVarValue(id, value);
+				});
+
+		OBSDataAutoRelease image =
+			obs_data_get_obj(event.data, "image");
+		SetTempVarValue("image.url_4x",
+				obs_data_get_string(image, "url_4x"));
+
+		OBSDataAutoRelease default_image =
+			obs_data_get_obj(event.data, "default_image");
+		SetTempVarValue("default_image.url_4x",
+				obs_data_get_string(default_image, "url_4x"));
+
+		OBSDataAutoRelease max_per_stream =
+			obs_data_get_obj(event.data, "max_per_stream");
+		SetTempVarValue("max_per_stream.is_enabled",
+				obs_data_get_bool(max_per_stream,
+						  "is_enabled"));
+		SetTempVarValue("max_per_stream.max_per_stream",
+				std::to_string(obs_data_get_int(max_per_stream,
+								"value")));
+
+		OBSDataAutoRelease max_per_user_per_stream =
+			obs_data_get_obj(event.data, "max_per_user_per_stream");
+		SetTempVarValue("max_per_user_per_stream.is_enabled",
+				obs_data_get_bool(max_per_user_per_stream,
+						  "is_enabled"));
+		SetTempVarValue(
+			"max_per_user_per_stream.max_per_user_per_stream",
+			std::to_string(obs_data_get_int(max_per_user_per_stream,
+							"value")));
+
+		OBSDataAutoRelease global_cooldown =
+			obs_data_get_obj(event.data, "global_cooldown");
+		SetTempVarValue("global_cooldown.is_enabled",
+				obs_data_get_bool(global_cooldown,
+						  "is_enabled"));
+		SetTempVarValue(
+			"max_per_user_per_stream.global_cooldown_seconds",
+			std::to_string(obs_data_get_int(max_per_user_per_stream,
+							"seconds")));
+	});
+}
+
+bool MacroConditionTwitch::CheckChannelRewardRedemptionEvents()
+{
+	return HandleMatchingSubscriptionEvents([this](const Event &event) {
+		SetVariableValue(event.ToString());
+		SetJsonTempVars(event.data,
+				[this](const char *id, const char *value) {
+					SetTempVarValue(id, value);
+				});
+		OBSDataAutoRelease rewardInfo =
+			obs_data_get_obj(event.data, "reward");
+		SetJsonTempVars(rewardInfo, [this](const char *nestedId,
+						   const char *value) {
+			const std::string id =
+				"reward." + std::string(nestedId);
+			SetTempVarValue(id, value);
+		});
+	});
+}
+
+bool MacroConditionTwitch::HandleMatchingSubscriptionEvents(
+	const std::function<void(const Event &)> &matchCb)
+{
+	if (!_eventBuffer) {
+		return false;
+	}
+
+	while (!_eventBuffer->Empty()) {
+		auto event = _eventBuffer->ConsumeMessage();
+		if (!event) {
+			continue;
+		}
+		if (_subscriptionID != event->id) {
+			continue;
+		}
+		matchCb(*event);
 
 		if (_clearBufferOnMatch) {
 			_eventBuffer->Clear();
@@ -618,16 +663,18 @@ bool MacroConditionTwitch::CheckCondition()
 	case Condition::CHARITY_CAMPAIGN_END_EVENT:
 	case Condition::SHIELD_MODE_START_EVENT:
 	case Condition::SHIELD_MODE_END_EVENT:
-	case Condition::POINTS_REWARD_ADDITION_EVENT:
-	case Condition::POINTS_REWARD_UPDATE_EVENT:
-	case Condition::POINTS_REWARD_DELETION_EVENT:
-	case Condition::POINTS_REWARD_REDEMPTION_EVENT:
-	case Condition::POINTS_REWARD_REDEMPTION_UPDATE_EVENT:
 	case Condition::USER_BAN_EVENT:
 	case Condition::USER_UNBAN_EVENT:
 	case Condition::USER_MODERATOR_ADDITION_EVENT:
 	case Condition::USER_MODERATOR_DELETION_EVENT:
 		return CheckChannelGenericEvents();
+	case Condition::POINTS_REWARD_ADDITION_EVENT:
+	case Condition::POINTS_REWARD_UPDATE_EVENT:
+	case Condition::POINTS_REWARD_DELETION_EVENT:
+		return CheckChannelRewardChangeEvents();
+	case Condition::POINTS_REWARD_REDEMPTION_EVENT:
+	case Condition::POINTS_REWARD_REDEMPTION_UPDATE_EVENT:
+		return CheckChannelRewardRedemptionEvents();
 	case Condition::STREAM_ONLINE_LIVE_EVENT:
 	case Condition::STREAM_ONLINE_PLAYLIST_EVENT:
 	case Condition::STREAM_ONLINE_WATCHPARTY_EVENT:
@@ -1206,25 +1253,41 @@ void MacroConditionTwitch::SetupTempVars()
 		setupTempVarHelper("is_user_input_required", ".reward");
 		setupTempVarHelper("should_redemptions_skip_request_queue",
 				   ".reward");
-		setupTempVarHelper("max_per_stream", ".reward");
-		setupTempVarHelper("max_per_user_per_stream", ".reward");
 		setupTempVarHelper("background_color", ".reward");
 		setupTempVarHelper("image", ".reward");
+		setupTempVarHelper("image.url_4x", ".reward");
 		setupTempVarHelper("default_image", ".reward");
-		setupTempVarHelper("global_cooldown", ".reward");
+		setupTempVarHelper("default_image.url_4x", ".reward");
 		setupTempVarHelper("cooldown_expires_at", ".reward");
 		setupTempVarHelper("redemptions_redeemed_current_stream",
 				   ".reward");
+		setupTempVarHelper("max_per_stream", ".reward");
+		setupTempVarHelper("max_per_stream.is_enabled", ".reward");
+		setupTempVarHelper("max_per_stream.max_per_stream", ".reward");
+		setupTempVarHelper("max_per_user_per_stream", ".reward");
+		setupTempVarHelper("max_per_user_per_stream.is_enabled",
+				   ".reward");
+		setupTempVarHelper(
+			"max_per_user_per_stream.max_per_user_per_stream",
+			".reward");
+		setupTempVarHelper("global_cooldown", ".reward");
+		setupTempVarHelper("global_cooldown.is_enabled", ".reward");
+		setupTempVarHelper("global_cooldown.global_cooldown_seconds",
+				   ".reward");
 		break;
 	case Condition::POINTS_REWARD_REDEMPTION_EVENT:
 	case Condition::POINTS_REWARD_REDEMPTION_UPDATE_EVENT:
-		setupTempVarHelper("id", ".reward");
+		setupTempVarHelper("id", ".redemption");
 		setupTempVarHelper("user_id", ".reward");
 		setupTempVarHelper("user_login", ".reward");
 		setupTempVarHelper("user_name", ".reward");
 		setupTempVarHelper("user_input", ".reward");
 		setupTempVarHelper("status", ".reward");
 		setupTempVarHelper("reward", ".reward");
+		setupTempVarHelper("reward.id", ".reward");
+		setupTempVarHelper("reward.title", ".reward");
+		setupTempVarHelper("reward.prompt", ".reward");
+		setupTempVarHelper("reward.cost", ".reward");
 		setupTempVarHelper("redeemed_at", ".reward");
 		break;
 	case Condition::USER_BAN_EVENT:
@@ -1387,7 +1450,7 @@ MacroConditionTwitchEdit::MacroConditionTwitchEdit(
 	  _tokens(new TwitchConnectionSelection()),
 	  _tokenWarning(new QLabel()),
 	  _channel(new TwitchChannelSelection(this)),
-	  _pointsReward(new TwitchPointsRewardWidget(this)),
+	  _pointsReward(new TwitchPointsRewardWidget(this, true)),
 	  _streamTitle(new VariableLineEdit(this)),
 	  _regexTitle(new RegexConfigWidget(parent)),
 	  _chatMesageEdit(new ChatMessageEdit(this)),
@@ -1425,10 +1488,8 @@ MacroConditionTwitchEdit::MacroConditionTwitchEdit(
 		this,
 		SLOT(ChatMessagePatternChanged(const ChatMessagePattern &)));
 	QWidget::connect(_category,
-			 SIGNAL(CategoreyChanged(const TwitchCategory &)), this,
-			 SLOT(CategoreyChanged(const TwitchCategory &)));
-	QWidget::connect(this, SIGNAL(TempVarsChanged()), window(),
-			 SIGNAL(SegmentTempVarsChanged()));
+			 SIGNAL(CategoryChanged(const TwitchCategory &)), this,
+			 SLOT(CategoryChanged(const TwitchCategory &)));
 	QWidget::connect(_clearBufferOnMatch, SIGNAL(stateChanged(int)), this,
 			 SLOT(ClearBufferOnMatchChanged(int)));
 
@@ -1480,7 +1541,6 @@ void MacroConditionTwitchEdit::ConditionChanged(int idx)
 	_entryData->SetCondition(static_cast<MacroConditionTwitch::Condition>(
 		_conditions->itemData(idx).toInt()));
 	SetWidgetVisibility();
-	emit TempVarsChanged();
 }
 
 void MacroConditionTwitchEdit::TwitchTokenChanged(const QString &token)
@@ -1572,7 +1632,7 @@ void MacroConditionTwitchEdit::RegexTitleChanged(const RegexConfig &conf)
 	updateGeometry();
 }
 
-void MacroConditionTwitchEdit::CategoreyChanged(const TwitchCategory &category)
+void MacroConditionTwitchEdit::CategoryChanged(const TwitchCategory &category)
 {
 	GUARD_LOADING_AND_LOCK();
 	_entryData->_category = category;
