@@ -439,7 +439,7 @@ TwitchTokenSettingsDialog::TwitchTokenSettingsDialog(
 			     "AdvSceneSwitcher.twitchToken.select",
 			     "AdvSceneSwitcher.twitchToken.add",
 			     "AdvSceneSwitcher.twitchToken.nameNotAvailable",
-			     parent),
+			     false, parent),
 	  _requestToken(new QPushButton(
 		  obs_module_text("AdvSceneSwitcher.twitchToken.request"))),
 	  _showToken(new QPushButton()),
@@ -456,7 +456,6 @@ TwitchTokenSettingsDialog::TwitchTokenSettingsDialog(
 	_currentTokenValue->setText(QString::fromStdString(settings._token));
 
 	_name->setReadOnly(true);
-	_showNameEmptyWarning = false;
 
 	QWidget::connect(_requestToken, SIGNAL(clicked()), this,
 			 SLOT(RequestToken()));
@@ -561,7 +560,9 @@ void TwitchTokenSettingsDialog::CheckIfTokenValid()
 
 void TwitchTokenSettingsDialog::ShowToken()
 {
-	SetButtonIcon(_showToken, ":res/images/visible.svg");
+	SetButtonIcon(_showToken, GetThemeTypeName() == "Light"
+					  ? ":res/images/visible.svg"
+					  : "theme:Dark/visible.svg");
 	_currentTokenValue->setEchoMode(QLineEdit::Normal);
 }
 
@@ -682,22 +683,33 @@ void TwitchTokenSettingsDialog::RequestToken()
 void TwitchTokenSettingsDialog::GotToken(const std::optional<QString> &value)
 {
 	_currentTokenValue->setText(value.value_or(""));
-	if (value.has_value()) {
-		_tokenStatus->setText(obs_module_text(
-			"AdvSceneSwitcher.twitchToken.request.success"));
-		_currentToken.SetToken(value.value().toStdString());
-		auto name = QString::fromStdString(_currentToken._name);
-		_name->setText(name);
-		_name->textEdited(name);
-		QMetaObject::invokeMethod(this, "NameChanged",
-					  Q_ARG(const QString &, name));
-		SetTokenInfoVisible(true);
-	} else {
+	if (!value) {
 		_tokenStatus->setText(obs_module_text(
 			"AdvSceneSwitcher.twitchToken.request.fail"));
 		_name->setText("");
 		SetTokenInfoVisible(false);
+		_requestToken->setEnabled(true);
+		return;
 	}
+
+	_currentToken.SetToken(value.value().toStdString());
+	auto name = QString::fromStdString(_currentToken._name);
+	if (name.isEmpty()) {
+		_tokenStatus->setText(obs_module_text(
+			"AdvSceneSwitcher.twitchToken.request.fail"));
+		_name->setText("");
+		SetTokenInfoVisible(false);
+		_requestToken->setEnabled(true);
+		return;
+	}
+
+	_tokenStatus->setText(obs_module_text(
+		"AdvSceneSwitcher.twitchToken.request.success"));
+	_name->setText(name);
+	_name->textEdited(name);
+	QMetaObject::invokeMethod(this, "NameChanged",
+				  Q_ARG(const QString &, name));
+	SetTokenInfoVisible(true);
 	_requestToken->setEnabled(true);
 }
 
