@@ -12,7 +12,7 @@ QObject *HighlightWidget(QWidget *widget, QColor startColor, QColor endColor,
 
 void SetHeightToContentHeight(QListWidget *list) {}
 
-void SetButtonIcon(QPushButton *button, const char *path) {}
+void SetButtonIcon(QAbstractButton *button, const char *path) {}
 
 int FindIdxInRagne(QComboBox *list, int start, int stop,
 		   const std::string &value, Qt::MatchFlags flags)
